
import datetime
import os
from gettext import gettext as _

import gtk
import dbus
import gc
from mayanna_base import Item, ItemSource, Topic
from mayanna_util import icon_factory, ToolMenuButton, ImplementMe
#from mayanna_empathy import EmpathySource
from mayanna_evolution import EvolutionSource


#
# Toolbar Items
#

class AccountMenu(gtk.Menu):
    def __init__(self):
        gtk.Menu.__init__(self)

        self.item_available = gtk.ImageMenuItem(_("Available"))
        self.item_available.set_image(icon_factory.load_image(gtk.STOCK_YES, gtk.ICON_SIZE_MENU))
        self.item_available.connect("activate", lambda w: self._set_status("available"))
        self.item_available.show()
        self.append(self.item_available)

        self.item_offline = gtk.ImageMenuItem(_("Offline"))
        self.item_offline.set_image(icon_factory.load_image(gtk.STOCK_CANCEL, gtk.ICON_SIZE_MENU))
        self.item_offline.connect("activate", lambda w: self._set_status("offline"))
        self.item_offline.show()
        self.append(self.item_offline)
        
        self.item_away = gtk.ImageMenuItem(_("Away"))
        self.item_away.set_image(icon_factory.load_image(gtk.STOCK_NO, gtk.ICON_SIZE_MENU))
        self.item_away.connect("activate", lambda w: self._set_status("away"))
        self.item_away.show()
        self.append(self.item_away)
        
        self.item_invisible = gtk.ImageMenuItem(_("Invisible"))
        self.item_invisible.set_image(icon_factory.load_image("status-invisible",
                                                              gtk.ICON_SIZE_MENU))
        self.item_invisible.connect("activate", lambda w: self._set_status("invisible"))
        self.item_invisible.show()
        self.append(self.item_invisible)

    def _show_accounts_editor(self):
        ImplementMe()

    def _set_status(self, status):
        pidgin_reader.set_global_status(status)


class AddPersonToolButton(gtk.ToolButton):
    __gsignals__ = {
        "clicked" : "override"
        }

    def __init__(self, tooltips):
        img = icon_factory.load_image("stock_person", gtk.ICON_SIZE_LARGE_TOOLBAR)
        gtk.ToolButton.__init__(self, img, _("New Person"))
        self.set_tooltip(tooltips, _("Add a new contact person"))

    def _pidgin_get_default_account_obj(self):
        acct = pidgin_dbus.PurpleAccountsFindConnected("", "")
        if not acct:
            acct = pidgin_dbus.PurpleAccountsFindAny("", "")
        return acct

    def do_clicked(self):
        try:
            acct = self._pidgin_get_default_account_obj()
            pidgin_dbus.PurpleBlistRequestAddBuddy(acct, "", "", "")
        except:
            pass #print " !!! Pidgin or Gaim2 D-BUS interface not available.  Is either Pidgin or Gaim2 running?"


class AvailableToolMenuButton(ToolMenuButton):
    def __init__(self, tooltips):
        ToolMenuButton.__init__(self, gtk.Image(), _("Status"))

        self.set_tooltip(tooltips, _("Set your online status"))
        self.set_menu(AccountMenu())
    
        # Connect to dbus signals to notify us of buddy status changes
        # NOTE: When going to 'invisible' there won't be an accountStatusChanged
        #       signal (tested with an xmpp account might be different on
        #       others)
        try:
            pidgin_reader.connect("status-changed", lambda x: self._status_changed())
            self._status_changed()
        except dbus.DBusException:
            pidgin_reader.reload_dbus()
            #print " !!! Pidgin D-BUS interface not available.  Is Pidgin running?"

    def _status_changed(self):
        status_str = pidgin_reader.get_global_status()

        icon = None
        if status_str == _("Available"):
            icon = gtk.STOCK_YES
        elif status_str == _("Offline"):
            icon = gtk.STOCK_CANCEL
        elif status_str == _("Away"):
            icon = gtk.STOCK_NO
        elif status_str == _("Invisible"):
            # FIXME: Need icon for invisible
            icon = "status-invisible"
        else:
            pass #print " !!! Unknown Pidgin status:", status_str

        if icon:
            self.get_icon_widget().set_from_pixbuf(
                icon_factory.load_icon(icon, gtk.ICON_SIZE_BUTTON))
        self.set_label(status_str.capitalize())


#
# Topic Implementation
#

class PeopleTopic(Topic):
    '''
    Lists recently talked to people, people online now, and all contacts, as
    well as listing out Gaim groups.  Placeholders for future
    Gmail/Friendster/LinkedIn contact integration.
    '''
    def __init__(self,prefs=None):
        Topic.__init__(self,
                       _("People"),
                       uri="topic://People",
                       icon="stock_people",
                       prefs=prefs)

        #from mayanna_running import RunningChats
        #self.set_running_source_factory(lambda: RunningChats())

        # Reload the sidebar when Pidgin's files change
        #pidgin_reader.connect("reload", lambda x: self.emit("reload"))

        # Reload the sidebar when Gaim's files change
        self.source_list =[]
        self.do_reload()

    def do_reload(self):
        
        self.source_list = [
                       #OnlineBuddySource(_("Online Now"), "im"),
                       #EverybodySource(_("Everybody"), "gnome-globe"),
                       #EmpathySource(_("Empathy"), "gnome-globe"),
                       EvolutionSource()
                       ]
        
        # Accounts in Pidgin/Gaim are mutually exclusive
        #acct_list = pidgin_reader.get_accounts()
        #for acct in acct_list:
            #self.source_list.append(acct)
        #del acct_list
        gc.collect()

    def get_hint_color(self):
        return gtk.gdk.color_parse("darkgreen")

    def get_toolbar_items(self, tooltips):
        tools = []

        btn = AddPersonToolButton(tooltips)
        btn.set_is_important(True)
        tools.append(btn)

        btn = AvailableToolMenuButton(tooltips)
        btn.set_is_important(True)
        tools.append(btn)

        return tools

