
from mayanna_engine.mayanna_util import *
from mayanna_engine.mayanna_computer import *
import datetime
import math
from xml.sax import saxutils
from gettext import gettext as _
import gmenu
import glob
import os
import pwd
import wnck
import gnomeapplet

import gobject
import gtk
import pango
import gconf
import gc
import gnomedesktop


class ZoomMenuToolItem(ToolMenuButton):
    __gsignals__ = {
        "zoom-changed" : (gobject.SIGNAL_RUN_LAST,
                          gobject.TYPE_NONE,
                          (gobject.TYPE_INT,)),
        "open-timeline" : (gobject.SIGNAL_RUN_FIRST,
                           gobject.TYPE_NONE,
                           ())
        }

    _zoomlist = [[_("Today"), 1],
                 [_("This week"), 7],
                 [_("Last 2 weeks"), 14],
                 [_("This month"), 30],
                 [_("Last 2 months"), 60],
                 [_("Last 3 months"), 90],
                 [_("Last 6 months"), 180]]
    _default_zoom = 7

    def __init__(self):
        img = gtk.Image()
        img.set_from_pixbuf(icon_factory.load_icon(gtk.STOCK_ZOOM_IN,
                                                   gtk.ICON_SIZE_SMALL_TOOLBAR))
        ToolMenuButton.__init__(self, img, "")
       # delimg
        self.zoom_level = self._default_zoom
        group = None
        menu = gtk.Menu()

        for name, days in self._zoomlist:
            group = self._make_menu_item(group, name, days)
            if days == self.zoom_level:
                group.set_active(True)
            menu.append(group)

        item = gtk.SeparatorMenuItem()
        menu.append(weakref(item))

        item = gtk.CheckMenuItem(_("Show timeline"))
        item.connect("activate", lambda w: self.emit("open-timeline"))
        menu.append(item)

        menu.show_all()

        self.set_menu(menu)
        self.show_all()
        
    def _item_toggled(self, w, name, num_days):
        if w.get_active():
            self.set_label(name)
            self.zoom_level = num_days
            self.emit("zoom-changed", num_days)

    def _make_menu_item(self, group, name, num_days):
        item = gtk.RadioMenuItem(group, name)
        item.connect("toggled", self._item_toggled, name, num_days)
        item.zoom_level = num_days
        return item

    def get_zoom_level(self):
        return self.zoom_level

    def get_zoom_level_list(self, floor = _default_zoom):
        return [days for name, days in self._zoomlist if days >= floor]

    def set_zoom_level(self, zoom_level):
        self.zoom_level = zoom_level
        for btn in self.get_menu().get_children():
            if hasattr(btn, "zoom_level") and btn.zoom_level == zoom_level:
                btn.set_active(True)

class TrayManagerBox(gtk.HBox):
    __gsignals__ = {
        'realize' : 'override'
        }

    def __init__(self):
        gtk.HBox.__init__(self, False, 0)

        from mayanna_engine.traymanager import TrayManager
        self.tray_mgr = TrayManager()
        self.tray_mgr.connect("tray_icon_added", self._add_tray_icon)
        self.tray_mgr.connect("tray_icon_removed", self._remove_tray_icon)

    def do_realize(self):
        self.chain()
        self.tray_mgr.manage_screen(gtk.gdk.screen_get_default())

    def _add_tray_icon(self, traymgr, icon):
        self.pack_start(icon, False, True, 0)

    def _remove_tray_icon(self, traymgr, icon):
        self.remove(icon)

#
# Toolbar Items
#

class UserMenu(gtk.Menu):
    def __init__(self):
        gtk.Menu.__init__(self)

        btn = gtk.ImageMenuItem(_("About Me"))
        btn.set_image(icon_factory.load_image("user-info", gtk.ICON_SIZE_SMALL_TOOLBAR))
        btn.connect("activate", lambda b: launcher.launch_command("gnome-about-me"))
        btn.show()
        self.append(btn)

        btn = gtk.ImageMenuItem(_("Switch User"))
        btn.set_image(icon_factory.load_image("gnome-lockscreen", gtk.ICON_SIZE_SMALL_TOOLBAR))
        btn.connect("activate",lambda w: launcher.launch_command(""))
        btn.show()
        self.append(btn)

    def _logout(self, btn, method):
        dialog = LogoutDialog(method)
        dialog.show()

    def _switch_user(self, button):
        gdmclient.new_login()

class UserToolMenuButton(ToolMenuButton):
    def __init__(self, tooltips):
        # Get the user's real name from /etc/passwd
        pwent = pwd.getpwuid(os.getuid())
        self.username = pwent.pw_gecos.split(",")[0] or pwent.pw_name
        name = self.username
        self.face_monitor = FileMonitor(os.path.expanduser("~/.face"))
        self.face_monitor.open()

        img = gtk.Image()
        self.face_monitor.connect("event", lambda m, u, ev, img: self._dot_face_changed, img)
        self._dot_face_changed(img)
        img.show()
        
        ToolMenuButton.__init__(self, img, name)
        self.set_tooltip(tooltips, _("Switch to another user"))
        self.set_menu(UserMenu())

    def _dot_face_changed(self, img):
        face = os.path.expanduser("~/.face")
        if not os.path.exists(face):
            face = "stock_person"
        img.set_from_pixbuf(icon_factory.load_icon(face, gtk.ICON_SIZE_SMALL_TOOLBAR))
    
class ShutdownToolButton(gtk.ToolButton):
    __gsignals__ = {
        "clicked" : "override"
        }

    def __init__(self, tooltips):
        img = icon_factory.load_image("gnome-shutdown", gtk.ICON_SIZE_SMALL_TOOLBAR)
        gtk.ToolButton.__init__(self, img, _("Shutdown..."))
        self.set_tooltip(tooltips, _("Shutdown or suspend this computer"))

    def do_clicked(self):
        os.popen("gnome-session-save --kill")
        
class HelpToolMenuButton(ToolMenuButton):
    def __init__(self, tooltips):
        img = icon_factory.load_image(gtk.STOCK_HELP, gtk.ICON_SIZE_SMALL_TOOLBAR)
        ToolMenuButton.__init__(self, img, _("Help"))

        menu = gtk.Menu()
        self.set_menu(menu)

        about = gtk.ImageMenuItem(_("About Gnome"))
        about.set_image(icon_factory.load_image(gtk.STOCK_ABOUT, gtk.ICON_SIZE_LARGE_TOOLBAR))
        about.connect("activate", lambda w: launcher.launch_command("gnome-about"))
        about.show()
        menu.append(about)
        
class VolumeToolMenuButton(ToolMenuButton):
    '''Dummy placeholder until this implements a volume control'''
    def __init__(self, tooltips):
        img = icon_factory.load_image("gnome-audio", gtk.ICON_SIZE_SMALL_TOOLBAR)
        ToolMenuButton.__init__(self, img, "")
        self.set_tooltip(tooltips, _("Volume: 75%"))

        menu = gtk.Menu()
        self.set_menu(menu)

        mi = gtk.MenuItem(_("Not Implemented"))
        mi.set_sensitive(False)
        mi.show()
        menu.append(mi)

class NetworkLocationToolMenuButton(ToolMenuButton):
    '''Dummy placeholder until this implements NetworkManager-applet behavior'''
    def __init__(self, tooltips):
        img = icon_factory.load_image(gtk.STOCK_NETWORK, gtk.ICON_SIZE_LARGE_TOOLBAR)
        ToolMenuButton.__init__(self, img, "")
        self.set_tooltip(tooltips, _("Change the network location"))

        menu = gtk.Menu()
        self.set_menu(menu)

        mi = gtk.MenuItem(_("Not Implemented"))
        mi.set_sensitive(False)
        mi.show()
        menu.append(mi)

#
# Toolbar Items
#

class NewFromTemplateDialog(gtk.FileChooserDialog):
    '''
    Dialog to create a new document from a template
    '''
    __gsignals__ = {
        "response" : "override"
        }

    def __init__(self, parent, name, source_uri):
        # Extract the template's file extension
        try:
            self.file_extension = name[name.rindex('.'):]
            name = name[:name.rindex('.')]
        except ValueError:
            self.file_extension = None
        self.source_uri = source_uri

        gtk.FileChooserDialog.__init__(self,
                                       _("New Document"),
                                       parent,
                                       gtk.FILE_CHOOSER_ACTION_SAVE,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                        gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT))
        self.set_current_name(name)
        self.set_current_folder(os.path.expanduser("~/Desktop"))
        self.set_do_overwrite_confirmation(True)
        self.set_default_response(gtk.RESPONSE_ACCEPT)

    def do_response(self, response):
        if response == gtk.RESPONSE_ACCEPT:
            file_uri = self.get_filename()

            # Add the file extension back unless the user did themselves
            if self.file_extension and not file_uri.endswith(self.file_extension):
                file_uri = "%s%s" % (self.source_uri, self.file_extension)

            # Create a new document from the template and display it
            try:
                if not self.source_uri:
                    # Create an empty file
                    f = open(file_uri, 'w')
                    f.close()
                else:
                    shutil.copyfile(self.source_uri, file_uri)
                launcher.launch_uri(file_uri)
            except IOError:
                pass

        self.destroy()


class NewDocumentMenuButton(ToolMenuButton):
    def __init__(self, tooltips):
        img = icon_factory.load_image("stock_new-template", gtk.ICON_SIZE_SMALL_TOOLBAR)

        ToolMenuButton.__init__(self, img, _("New Document"))
        self.set_tooltip(tooltips, _("Create a new document"))
        
        # Monitor the templates directory
        self.template_monitor = FileMonitor(os.path.expanduser("~/Templates"))
        self.template_monitor.connect("created", self._template_changed)
        self.template_monitor.connect("deleted", self._template_changed)
        self.template_monitor.open()
        self._template_changed(None, None)

    def _template_changed(self, monitor, uri):
        menu = gtk.Menu()
        self._add_templates(menu)
        self.set_menu(menu)

    def _add_template_item(self, menu, name, uri, icon_name):
        menu_item = gtk.ImageMenuItem(name)
        menu_item.set_image(icon_factory.load_image(icon_name, gtk.ICON_SIZE_SMALL_TOOLBAR))
        menu_item.show()
        menu_item.connect("activate", self._show_new_from_template_dialog, name, uri)
        return menu_item

    def _add_templates(self, menu):
        # Get all the templates
        template_dir = os.path.expanduser("~/Templates")
        try:
            templates = os.listdir(template_dir)
        except:
            templates = []

        for template in templates:
            item = FileItem(os.path.join(template_dir, template))
            menu.append(self._add_template_item(menu,
                                                item.get_name(),
                                                item.get_uri(),
                                                item.get_icon(gtk.ICON_SIZE_SMALL_TOOLBAR)))
        if len(templates) == 0:
            empty_item = gtk.MenuItem(_("No templates available"))
            empty_item.set_sensitive(False)
            empty_item.show()
            menu.append(empty_item)

        sep = gtk.SeparatorMenuItem()
        sep.show()
        menu.append(sep)

        menu.append(self._add_template_item(menu, _("Empty File"), None, "stock_new"))

        sep = gtk.SeparatorMenuItem()
        sep.show()
        menu.append(sep)

        # Add a link to the templates directory
        mi = gtk.ImageMenuItem(_("Templates"))
        mi.set_image(icon_factory.load_image("gnome-fs-directory", gtk.ICON_SIZE_SMALL_TOOLBAR))
        mi.show()
        mi.connect("activate", lambda mi, uri: launcher.launch_uri(uri), template_dir)
        menu.append(mi)

    def _show_new_from_template_dialog(self, b, name, uri):        
        dlg = NewFromTemplateDialog(b.get_toplevel(), name, uri)
        dlg.show()


class PlacesMenu(gtk.Menu):
    def __init__(self):
        gtk.Menu.__init__(self)

        # Include all the mounted drives
        computer = mayanna_get_topic_for_uri("topic://System")
        device_source = computer.get_source_for_uri("source://Devices")
        for item in device_source.get_items():
            if isinstance(item, DriveItem) and item.get_is_mounted():
                self.append(self._add_place_item(item.get_name(),
                                                 item.get_uri(),
                                                 item.get_icon(gtk.ICON_SIZE_LARGE_TOOLBAR)))
           # delitem
        sep = gtk.SeparatorMenuItem()
        sep.show()
        self.append(sep)

        # Include all the Gtk bookmarks
        for uri, name, mime, icon in places.get_places():
            self.append(self._add_place_item(name, uri, icon))
           # deluri, name, mime, icon

        sep = gtk.SeparatorMenuItem()
        sep.show()
        self.append(sep)

        # Add the trash
        if trash_monitor.is_empty():
            self.append(self._add_place_item(_("Trash"), "trash://", "gnome-fs-trash-empty"))
        else:
            self.append(self._add_place_item(_("Trash"), "trash://", "gnome-fs-trash-full"))

    def _add_place_item(self, name, uri, icon_name = "gnome-fs-directory"):
        menu_item = gtk.ImageMenuItem(name)
        menu_item.set_image(icon_factory.load_image(icon_name, gtk.ICON_SIZE_LARGE_TOOLBAR))
        menu_item.show()
        menu_item.connect("activate", lambda mi, uri: launcher.launch_uri(uri), uri)
        return menu_item


class PlacesMenuButton(ToolMenuButton):
    def __init__(self, tooltips):
        img = icon_factory.load_image("gnome-fs-bookmark", gtk.ICON_SIZE_LARGE_TOOLBAR)
        ToolMenuButton.__init__(self, img, _("Places"))
        self.set_tooltip(tooltips, _("Open commonly used locations"))
        self.set_menu(PlacesMenu())


class TrashButton(gtk.ToolButton):
    def __init__(self, tooltips):
        img = icon_factory.load_image("gnome-fs-trash-full", gtk.ICON_SIZE_LARGE_TOOLBAR)
        gtk.ToolButton.__init__(self, img, _("Trash"))
        self.set_tooltip(tooltips, _("Open the deleted files trashcan"))
        self.connect("clicked", lambda x: launcher.launch_uri("trash://"))


class PrintersMenuButton(gtk.ToolButton):
    __gsignals__ = {
        "clicked" : "override"
        }

    def __init__(self, tooltips):
        img = icon_factory.load_image(gtk.STOCK_PRINT, gtk.ICON_SIZE_LARGE_TOOLBAR)
        gtk.ToolButton.__init__(self, img, _("Printers"))
        self.set_tooltip(tooltips, _("View printers available for document #printing"))

    def do_clicked(self):
        topic = mayanna_get_topic_for_uri("topic://Computer")
        


        
class FriendlyClock(gtk.ToggleButton):
    __gsignals__ = {
        'toggled' : 'override'
        }
    
    def __init__(self):
        gtk.ToggleButton.__init__(self)
        self.set_relief(gtk.RELIEF_NONE)
        self.set_resize_mode(gtk.RESIZE_IMMEDIATE)

        KillFocusPadding(self, "clock-applet-button")

        self.label = gtk.Label("XXX")
        self.label.set_use_markup(True)
        self.label.set_justify(gtk.JUSTIFY_CENTER)
        self.label.show()
        self.add(self.label)

        gobject.timeout_add(1000, self._update_time)
        self._update_time()

        self.popup = None

    def _update_time(self):
        now = datetime.datetime.now()
        day = now.strftime("%A")
        date = now.strftime("%B %d %Y")
        time =  now.strftime("%I:%M:%S %p")
        now_str = "<span font_family='monospace'>\n %s %s %s \n</span>" % (day, date, time)
        if self.label.get_text() != now_str:
            self.label.set_markup(now_str)
        return True

    class CalendarPopup(gtk.Window):
        def __init__(self):
            gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
            self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_UTILITY)
            self.set_wmclass("Mayanna", "Mayanna")
            self.set_decorated(False)
            self.set_resizable(False)
            self.set_keep_above(True)
            self.stick()
            self.set_title(_("Calendar"))

            frame = gtk.Frame()
           #frame.set_shadow_type(gtk.SHADOW_OUT)
            frame.show()
            self.add(frame)

            vbox = gtk.VBox(False, 6)
            vbox.set_border_width(6)
            vbox.show()
            frame.add(vbox)

            self.cal = gtk.Calendar()
            self.cal.set_display_options(self.cal.get_display_options() |
                                         gtk.CALENDAR_SHOW_WEEK_NUMBERS)
            self.cal.show()
            vbox.pack_start(self.cal, True, False, 0)

    def _close_on_escape(self, w, ev):
        if ev.keyval == gtk.gdk.keyval_from_name("Escape"):
            self.set_active(False)
            return True
        return False

    def _position_popup(self, popup):
        x, y = self.window.get_origin()
        sx,sy = self.window.get_size()
        x -= sx
        #y += sy
        w, h = popup.size_request()
        
        popup.move(x, y)
        popup.set_gravity(gtk.gdk.GRAVITY_SOUTH_EAST)

    def do_toggled(self):
        if self.get_active():
            self.popup = FriendlyClock.CalendarPopup()
            self.popup.connect("delete-event", lambda w, ev: self.set_active(False))
            self.popup.connect("key-press-event", self._close_on_escape)

            self._position_popup(self.popup)
            self.popup.present()
        elif self.popup:
            self.popup.destroy()
            self.popup = None

    def get_tooltip(self):
        now = datetime.datetime.now()
        day = now.strftime("%A %B %e, %Y")
        return "%s\n<span size='smaller'>No Appointments</span>" % (day)
    
class TaskbarItem(gtk.ToggleButton):
    def __init__(self, appw):
        
        gtk.ToggleButton.__init__(self)
        self.appw = appw
        
        self.appw = appw
        
        self.connect("button-press-event", self.on_button_press)
        
        self.appw.get_screen().connect("window-closed", self.winclose_event)
        self.appw.connect("workspace-changed", self.winspace_event)
        
        self.set_alignment(0,0.5)
        
        self.set_relief(gtk.RELIEF_NONE)
        self.boxbg = gtk.Image()
        self.boxbg.set_from_file("tile.xpm")
        
        self.image = gtk.Image()
        self.image.set_from_pixbuf(appw.get_mini_icon())
        self.image.set_pixel_size(16)
        self.image.show()
        
        label = gtk.Label()
        label = gtk.Label(self.appw.get_name())
        label.set_ellipsize(pango.ELLIPSIZE_END)
        label.set_justify(gtk.JUSTIFY_LEFT)
        label.show()
        
        hbox = gtk.HBox(False,-1)
        hbox.pack_start(self.image, False, False, 0)
        hbox.pack_start(label, True, True, 12)
        hbox.show()
        #self.set_size_request(-1,-1)
        self.add(hbox)
        
        
        self.show()
            
        
    def on_button_press(self, widget, event):
        self.appw.set_icon_geometry(self.window.get_position()[0],self.window.get_position()[1],self.window.get_size()[0],self.window.get_size()[1])
        if  event.button == 1:
            if self.appw.is_minimized():
                self.appw.unminimize(event.time)
                self.set_active(False)
            else:
                self.appw.minimize()
                self.set_active(True)
        elif event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
            mnuw = wnck.create_window_action_menu(self.appw)
            mnuw.popup(None, None, None, event.button, event.time, event)
            
    def winclose_event(self, screen, window):
        if self.appw.get_xid() == window.get_xid():
            self.destroy()
            return True
        return False
    
    def winspace_event(self, window):
        if window.is_pinned():
            self.window.stick()
        else:
            self.window.unstick()
        print "do something"
    
        self.appw = appw
        
        self.appw.set_icon_geometry(self.window.get_position()[0],self.window.get_position()[1],self.window.get_size()[0],self.window.get_size()[1])
        
        self.window.add_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.window.add_events(gtk.gdk._2BUTTON_PRESS)
        self.window.connect("button-press-event", self.on_button_press)
        
        self.appw.get_screen().connect("window-closed", self.winclose_event)
        self.appw.connect("workspace-changed", self.winspace_event)
        
        
        self.boxbg = gtk.Image()
        self.boxbg.set_from_file("tile.xpm")
        
        self.image = gtk.Image()
        self.image.set_from_pixbuf(appw.get_icon())
        self.image.show()
        
        self.window.window.set_back_pixmap(self.boxbg.get_pixbuf().render_pixmap_and_mask()[0] , False) 
        
        self.window.show()

            
class TaskBarBox(gtk.VBox):
    def __init__(self):
        gtk.VBox.__init__(self)    
        
        
        self.set_border_width(5)
        self.frame1 = gtk.Frame()
        self.alignment1 = gtk.Alignment(0.5,0.5,1.0,1.0)
        self.label1 = gtk.Label() 
        label="<b>  " +" Tasks" + "  </b>"
        self.label1.set_markup(label)
        self.frame1.set_label_align(0.5, 0.5)
        self.frame1.add(self.alignment1)
        self.frame1.set_label_widget(self.label1)
        
       #self.frame1.set_shadow_type(gtk.SHADOW_OUT)
        
        screen = wnck.screen_get_default()
        screen.force_update()
        windows = screen.get_windows_stacked()
        
        self.scrolled_window = gtk.ScrolledWindow()
        self.scrolled_window.set_border_width(4)
        self.scrolled_window.set_size_request(-1,200)
        #self.scrolled_window.set_resizable(True)

        self.scrolled_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.scrolled_window.show()
        self.alignment1.add(self.scrolled_window)
        
        self.vbox = gtk.VBox()
        
        self.scrolled_window.add_with_viewport(self.vbox)
        
        screen = wnck.screen_get_default()
        screen.force_update()
        screen.connect("window-opened", self.spawnw)
        for w in screen.get_windows():
            self.spawnw(screen, w)

        self.pack_start(self.frame1,True,True)
        self.show_all()


    def spawnw(self,screen, window):
        if window.get_window_type() == 0:
             if window.get_window_type()  in (0, 7, 8):
                item = TaskbarItem(window)
                item.show()
                self.vbox.pack_start(item,False,False)
            
