import datetime
import math
from xml.sax import saxutils
from gettext import gettext as _
import gmenu
import glob
import os
import pwd
import gnomeapplet

import gobject
import gtk
import pango
import gconf
import gc
import gnomedesktop

import wnck


import cairo
try:
    import iconentry
except ImportError:
    pass


try:
    import cairo
except ImportError: 
    pass
        
class DockWindow(gtk.Window):
    __gsignals__ = {
        'realize' : 'override',
        'size-allocate' : 'override',
        }

    def __init__(self, edge_gravity):
        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)

        self.set_wmclass("gnome-panel", "Gnome-panel")
        type = gtk.gdk.WINDOW_TYPE_HINT_SPLASHSCREEN
        self.set_type_hint(type)
        self.set_skip_pager_hint(True)
        self.set_skip_taskbar_hint(True)
        self.stick()
        #self.set_opacity(0.8)
        ### Uncomment to have icons grow upwards.
        ### FIXME: This trips metacity bugs and makes the window actually move position
        #self.set_gravity(gtk.gdk.GRAVITY_SOUTH_EAST)

        self.edge_gravity = edge_gravity
        self.set_strut = True
        
        if edge_gravity in (gtk.gdk.GRAVITY_EAST, gtk.gdk.GRAVITY_WEST):
            self.content = gtk.VBox (False, 0)
        else:
            self.content = gtk.HBox (False, 0)
        self.content.show()

        self.content_align = gtk.Alignment(xscale=1.0, yscale=1.0)
        self.content_align.add(self.content)
        self.content_align.show()
        self.add(self.content_align)

    def get_content(self):
        return self.content

    def get_content_alignment(self):
        return self.content_align

    def get_edge_gravity(self):
        return self.edge_gravity

    def get_win_placement(self):
        '''This will place the window on the edge corresponding to the edge gravity'''
        
        width, height = self.size_request()
        geom = self.get_screen().get_monitor_geometry(0)
        eg = self.edge_gravity
            
        if eg in (gtk.gdk.GRAVITY_SOUTH, gtk.gdk.GRAVITY_NORTH):
            x = (geom.width / 2) - (width / 2)
        elif eg == gtk.gdk.GRAVITY_EAST:
            x = geom.width - width
        elif eg == gtk.gdk.GRAVITY_WEST:
            x = 0

        if eg in (gtk.gdk.GRAVITY_EAST, gtk.gdk.GRAVITY_WEST):
            y = (geom.height / 2) - (height / 2)
        elif eg == gtk.gdk.GRAVITY_SOUTH:
            y = geom.height - height
        elif eg == gtk.gdk.GRAVITY_NORTH:
            y = 0

        return [geom.x + x, geom.y + y] # Compensate for multiple monitors

    def move_to_position(self):
        if self.window:
            ### FIXME: Apparantly other people who are not me don't need this
            self.window.set_override_redirect(True)
            apply(self.move, self.get_win_placement())
            self.window.set_override_redirect(False)

    def do_realize(self):
        # FIXME: chain fails for some reason
        #ret = self.chain()
        ret = gtk.Window.do_realize(self)
        self.move_to_position()
        return ret

    def set_wm_strut(self, val):
        self.set_strut = val
        self.queue_resize_no_redraw()

    def do_set_wm_strut(self):
        '''
        Set the _NET_WM_STRUT window manager hint, so that maximized windows
        and/or desktop icons will not overlap this window\'s allocatied area.
        See http://standards.freedesktop.org/wm-spec/latest for details.
        '''
        if self.window:
            if self.set_strut:
                # values are left, right, top, bottom
                propvals = [0, 0, 0, 0]

                geom = self.get_screen().get_monitor_geometry(0)
                eg = self.get_edge_gravity()
                x, y = self.window.get_origin()
                alloc = self.allocation

                if eg == gtk.gdk.GRAVITY_WEST:
                    propvals[0] = alloc.width + x
                elif eg == gtk.gdk.GRAVITY_EAST and x != 0:
                    propvals[1] = geom.width - x
                elif eg == gtk.gdk.GRAVITY_NORTH:
                    propvals[2] = alloc.height + y
                elif eg == gtk.gdk.GRAVITY_SOUTH and y != 0:
                    propvals[3] = geom.height - y

                # tell window manager to not overlap buttons with maximized window
                self.window.property_change("_NET_WM_STRUT",
                                            "CARDINAL",
                                            32,
                                            gtk.gdk.PROP_MODE_REPLACE,
                                            propvals)
            else:
                self.window.property_delete("_NET_WM_STRUT")

        return False

    def do_size_allocate(self, alloc):
        # FIXME: chain fails for some reason
        #ret = self.chain(alloc)
        ret = gtk.Window.do_size_allocate(self, alloc)
        self.move_to_position()

        # Setting _NET_WM_STRUT too early can confuse the WM if the window is
        # still located at 0,0, so do it in an idle.
        gobject.idle_add(self.do_set_wm_strut)
        
        return ret


class EdgeWindow(DockWindow):
    __gsignals__ = {
        'size-request' : 'override'
        }

    def __init__(self, edge_gravity):
        DockWindow.__init__(self, edge_gravity)
        self.set_keep_above(True)
        self.set_wm_strut(True)

    def do_size_request(self, req):
        ret = self.chain(req)

        # Give some whitespace
        geom = self.get_screen().get_monitor_geometry(0)

        ### Uncomment for a more stable bar size
        # If centered to a side, take 85% of available size
        req.height = geom.height * 1
        req.width = geom.width / 6

        return ret


class CairoWindow:
    def __init__(self):#,w,h,ix,iy,iw,ih):
        # Create the window itself
        self.window = EdgeWindow(gtk.gdk.GRAVITY_EAST)
        #self.window.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_SPLASHSCREEN)
        #self.window.set_default_size(w,h)
        #Set window profile
        
        #self.ev_box2 = gtk.EventBox()
        #self.ev_box2.set_above_child (0)
        #self.ev_box2.set_visible_window(0)
        #self.window.add(self.ev_box2)
        #Create the image surfaces
        self.icon1=cairo.ImageSurface
        self.icon2=cairo.ImageSurface
        self.icon3=cairo.ImageSurface
        self.icon4=cairo.ImageSurface
        
        self.screen_changed(self.window)
        #self.window.show_all()

        self.iconopacity = [0,0,0,0]
        self.intrans = True 
        
    def transition(self,step, speed, rate, termination_event):
        self.step = step
        if self.intrans == False:
            self.intrans = True
            # Add timer
            gobject.timeout_add(speed,self.updatefade, termination_event, rate)
        
    def updatefade(self, termination_event, rate):
        for i in range(0,len(self.iconopacity)):
            self.iconopacity[i] = self.iconopacity[i] + (rate*self.step[i])
            if (self.iconopacity[i]<=0 or self.iconopacity[i]>=1) and self.step[i]!=0:
                self.step[i]=0
                self.iconopacity[i]=int(self.iconopacity[i])
        self.Redraw()
        if self.step==[0,0,0,0]:
            self.intrans = False
            if termination_event:
                termination_event()
            return False
        return True
    
    def show_window(self):
        self.window.show()
        
    def show_all(self):
        self.window.show_all()

    def hide_window(self):
        self.window.hide()
        
    def screen_changed(self,widget):
        # Screen change event
        screen = widget.get_screen()
        colormap = screen.get_rgba_colormap()
        widget.set_colormap(colormap)
    
    def Repaint(self,widget,event):
        self.Redraw()
    
    def Redraw(self):
        if self.window.window:
            self.cr = self.window.window.cairo_create()
            self.cr.set_source_rgba(1.0, 1.0, 1.0, 0.0)
            self.cr.set_operator(cairo.OPERATOR_SOURCE) #DEST_ATOP
            #self.surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 200, 100)
            self.cr2 = cairo.Context(self.surface)
            
            '''
            if isinstance(self.icon1,cairo.Surface):
                self.cr2.set_source_surface(self.icon1, 0, 0)
                self.cr2.paint_with_alpha(self.iconopacity[0])
            if isinstance(self.icon2,cairo.Surface):
                self.cr2.set_source_surface(self.icon2, self.ix, self.iy)                
                self.cr2.paint_with_alpha(self.iconopacity[1])
            if isinstance(self.icon3,cairo.Surface):
                self.cr2.set_source_surface(self.icon3, 0, 0)
                self.cr2.paint_with_alpha(self.iconopacity[2])
            if isinstance(self.icon4,cairo.Surface):
                self.cr2.set_source_surface(self.icon4, 0, 0)
                self.cr2.paint_with_alpha(self.iconopacity[3])
            '''
            #self.cr.set_source_surface(self.surface, 0, 0)
            self.cr.paint()
        
    def move(self,x,y):
        # Relocate the window
        self.window.move(x,y)

    def resize(self,w,h):
        self.window.resize(w,h)

    def resizematch(self):
        self.w = self.icon1.get_width()
        self.h = self.icon1.get_height()
        self.window.resize(self.w,self.h)
    
    def destroy(self,event):
        # Remove the window
        self.window.destroy()
        gtk.main_quit()

    def update(self):
        # Update the screen profile on request
        self.screen_changed(self.window)
        
    def updateimage(self,index,imagefile):
        if imagefile.endswith(".png"):
            self.temp = cairo.ImageSurface.create_from_png(imagefile)
        else:
            # All .face files will load through this routine, even though they may be 'png' format
            self.temppixbuf = gtk.gdk.pixbuf_new_from_file(imagefile)
            # Resize the image and create the cairo surface to be used
            self.temp = cairo.ImageSurface(cairo.FORMAT_ARGB32, self.iw, self.ih)
            self.cr2 = cairo.Context(self.temp)
            w = float(self.iw)/float(self.temppixbuf.get_width())
            h = float(self.ih)/float(self.temppixbuf.get_height())
            self.cr2.scale(w,h)
            self.ct = gtk.gdk.CairoContext(self.cr2)
            self.ct.set_source_pixbuf(self.temppixbuf,0,0)
            self.ct.paint()
    
            # Update the graphic being displayed (first image has size priority)
        if index==0:
            self.icon1 = self.temp
        if index==1:
            self.icon2 = cairo.ImageSurface(cairo.FORMAT_ARGB32, self.w, self.h)
            self.cr2 = cairo.Context(self.icon2)
            w = float(self.iw)/float(self.temp.get_width())
            h = float(self.ih)/float(self.temp.get_height())
            self.cr2.scale(w,h)
            self.cr2.set_source_surface(self.temp, 0, 0)
            self.cr2.paint()
        if index==2:
            self.icon3 = self.temp
        if index==3:
            self.icon4 = self.temp
        self.Redraw()
