
from mayanna_engine.mayanna_applications import ApplicationsTopic
from mayanna_engine.mayanna_library import DocumentsTopic
from mayanna_engine.mayanna_people import PeopleTopic
from mayanna_engine.mayanna_computer import ComputerTopic
from mayanna_engine.mayanna_favorite import FavoritesTopic

from mayanna_engine.mayanna_base import ItemSource
from mayanna_engine.mayanna_computer import SettingsSource
from mayanna_engine.mayanna_util import *
from mayanna_extra_widgets import  *
from mayanna_gui_helpers import *
from gettext import gettext as _

class TopicButtonMod(gtk.ToggleButton):
    __gsignals__ = {
        'realize' : 'override',
        }

    def __init__(self, topic):
        gtk.ToggleButton.__init__(self)
        self.set_property("can-default", True)
        self.set_property("can-focus", True)
        self.set_border_width(0)
        self.set_name(topic.name)

        self.topic = topic
        icon = self.topic.get_icon(18)
        icon = icon_factory.load_image (icon, 18)
        self.set_image(icon)
        self.set_label(self.topic.get_name())
        self.set_relief(gtk.RELIEF_HALF)
        del icon
        gc.collect()
      
    def do_realize(self):
        self.chain()
        
        
        color = self.topic.get_hint_color()
        '''
        new = color_average_widget_state(color, self, gtk.STATE_NORMAL)
        self.modify_bg(gtk.STATE_NORMAL, color)
        '''
        new = color_average_widget_state(color, self, gtk.STATE_ACTIVE)
        self.modify_bg(gtk.STATE_ACTIVE, new)
        '''
        new = color_average_widget_state(color, self, gtk.STATE_PRELIGHT)
        self.modify_bg(gtk.STATE_PRELIGHT, new)
        '''
        


    def _map(self, win, ev):
        self.set_active(True)

    def _unmap(self, win, ev):
        self.set_active(False)
    
class ItemIconView(gtk.IconView):
    '''
    Icon view which displays Items in the style of the Nautilus horizontal mode,
    where icons are right aligned and each column is of a uniform width.  Also
    handles opening an item and displaying the item context menu.
    '''
    def __init__(self):
        gtk.IconView.__init__(self)
        self.set_orientation(gtk.ORIENTATION_HORIZONTAL)

        self.idle_load_id = None
        self.reload_handlers = {}

        # PyGtk before 2.8.4 doesn't expose GtkIconView's CellLayout interface
        self.use_cells = isinstance(self, gtk.CellLayout)
        if self.use_cells:
            # Pack the renderers manually, since GtkIconView layout is very buggy.
            self.icon_cell = gtk.CellRendererPixbuf()
            self.icon_cell.set_property("yalign", 0.0)
            self.icon_cell.set_property("xalign", 0.0)
            self.pack_start(self.icon_cell, expand=True)
            self.add_attribute(self.icon_cell, "pixbuf", 1)

            self.text_cell = gtk.CellRendererText()
            self.text_cell.set_property("wrap-mode", pango.WRAP_WORD_CHAR)
            self.text_cell.set_property("yalign", 0.0)
            self.text_cell.set_property("xalign", 0.0)
            self.pack_start(self.text_cell, expand=True)
            self.add_attribute(self.text_cell, "markup", 0)
        else:
            self.set_markup_column(0)
            self.set_pixbuf_column(1)
            self.set_item_width(230)

        self.set_margin(12)
        self.set_spacing(4)
        self.set_selection_mode(gtk.SELECTION_MULTIPLE)
        self.connect("item-activated", self._open_item)
        self.connect("button-press-event", self._show_item_popup)
        self.connect("button-release-event", self._button_release)
        self.connect("drag-data-get", self._item_drag_data_get)
        self.connect("motion-notify-event", self._motion_notify)
        self.enable_model_drag_source(0,
                                      [("text/uri-list", 0, 100)],
                                      gtk.gdk.ACTION_LINK | gtk.gdk.ACTION_COPY)

        self.click_policy = gconf_bridge.get("click_policy")
        gconf_bridge.connect("changed::click_policy", lambda gb: self._click_policy_changed())

        self.nautilus_gconf_notify_id = None
        if self.click_policy == "nautilus":
            self._connect_nautilus_click_policy()

        bookmarks.connect("reload", lambda b: self._reload_visible_items())

        self.highlight_path = None
        self.highlight_normal_icon = None
        self.highlight_timeout = 0
        
        self.store = gtk.ListStore(gobject.TYPE_STRING,    # 0: Markup text
                              gtk.gdk.Pixbuf,         # 1: Pixbuf
                              gobject.TYPE_PYOBJECT,  # 2: Item
                              gobject.TYPE_BOOLEAN)   # 3: Visible
        
        self.store.set_sort_func(2, self._compare_item_name)
        self.store.set_sort_column_id(2, gtk.SORT_ASCENDING)
        
        self.filterstore = self.store.filter_new()
        self.filterstore.set_visible_column(3)

        # IconView bug doesn't send selection-changed on set_model
        self.unselect_all()
        self.set_model(self.filterstore) # Set the model
        

    def _connect_nautilus_click_policy(self):
        nautilus_dir = "/apps/nautilus/preferences/"
        self.nautilus_click_policy_key = nautilus_dir + "click_policy"

        self.gconf_client = gconf.client_get_default()
        self.gconf_client.add_dir(nautilus_dir[:-1], gconf.CLIENT_PRELOAD_NONE)
        self.nautilus_gconf_notify_id = \
               self.gconf_client.notify_add(self.nautilus_click_policy_key,
                                            self._nautilus_click_policy_changed)

        value = self.gconf_client.get(self.nautilus_click_policy_key)
        if value:
            self.click_policy = value.get_string()
        else:
            self.click_policy = "double"

    def _disconnect_nautilus_click_policy(self):
        if self.nautilus_gconf_notify_id:
            self.gconf_client.notify_remove(self.nautilus_gconf_notify_id)
            self.nautilus_gconf_notify_id = None

    def _nautilus_click_policy_changed(self, client, cnxn_id, entry, data=None):
        value = self.gconf_client.get(self.nautilus_click_policy_key)
        if value:
            self.click_policy = value.get_string()

    def _compare_item_name(self, model, iter1, iter2, user_data = None):
        '''
        Item comparison: special items come first, followed by case insensitive
        name comparison.
        '''
        item1 = model.get_value(iter1, 2)
        item2 = model.get_value(iter2, 2)
        return not item1 and 1 or \
               not item2 and -1 or \
               cmp(not item1.is_special(), not item2.is_special()) or \
               cmp((item1.get_name() or "").lower(), (item2.get_name() or "").lower())

    def load_items(self, items, ondone_cb = None):
        '''
        Creates a new list store to load with item data, then sets up an idle
        timeout generator to fill it.
        '''
        gc.collect()
        # Clean up existing item reload connections
        for item, handler in self.reload_handlers.iteritems():
            item.handler_disconnect(handler)
            del handler, item
        self.reload_handlers.clear()
        self.store.clear()
        
        

        # Clear highlight state
        self.highlight_path = None
        self.highlight_normal_icon = None
        if self.highlight_timeout != 0:
            gobject.source_remove(self.highlight_timeout)
        if self.idle_load_id:
            gobject.source_remove(self.idle_load_id) # Cancel pending load
        self.idle_load_id = \
            gobject.timeout_add(0, self._idle_load_items(items, ondone_cb).next)
        del items
        gc.collect()
        
    def _idle_load_items(self, items, ondone_cb):
        '''
        Loads one item at a time, yielding True if there is more to load, and
        setting the IconView model and yielding False when finished.
        '''
        
        for i in items:
            iter = self.store.append((None, None, i, i.get_is_user_visible()))
            self.reload_handlers[i] = i.connect("reload", self._reload_item, iter)
           # deliter
            yield True # Keep going
        
        gc.collect()
       # delitems
        
        self._reload_visible_items()

        if ondone_cb:
            ondone_cb()
        
        del items
        gc.collect()
        yield False # All done
        
    def _update_sizing(self, icon_width):
        self.icon_cell.set_fixed_size(icon_width, -1) # Reset icon sizing
        wrap_width = 150 - icon_width - 4
        self.text_cell.set_property("wrap-width", wrap_width)
        del icon_width  
        gc.collect()     
        
        
    def _reload_item(self, item, iter):
        
        if self.store.iter_is_valid(iter):
            icon_size = self._set_item(item, iter)
            self._update_sizing(icon_size)
            del icon_size
        
        gc.collect()
        
        
    def _reload_visible_items(self):
        
        self.store = self.get_model()
        if isinstance(self.store, gtk.TreeModelFilter):
            # Get a writable model if a filter model is set
            self.store = self.store.get_model()

        max_size = 0
        for row in self.store:
            try:
                icon_size = self._set_item(row[2], row.iter)
                max_size = max(icon_size, max_size)
            except:
                pass
        self._update_sizing(max_size)
        del max_size
        

    def _set_item(self, item, iter):
        
        if not item.get_is_user_visible():
            self.store.set(iter, 3, False) # Hide the item
            return 0

        name = item.get_name_markup()
        largename = "<span size='large'>%s</span>" % name
        smallname = "<span size='small'>%s</span>" % name
        comment = "<span size='small'>%s</span>" % item.get_comment_markup()

       # Size based on number of visible items
        item_cnt = len(self.store)
        text = name + "\n"  + comment
        icon_size = 24

        try:
            icon = item.get_icon(icon_size)
            # Bound returned width to height * 2
            icon_width = min(icon.get_width(), icon.get_height() * 2)
        except (AssertionError, AttributeError):
            icon = None
            icon_width = 0

        # Update text, icon, and visibility
        self.store.set(iter, 0, text, 1, icon, 3, True)

        # Return the icon width used for sizing other records
        del icon,name,largename,smallname,comment,text
        gc.collect()
        return icon_width

    def _open_item(self, view, path):
        model = view.get_model()
        model.get_value(model.get_iter(path), 2).open()
        del model

    def _deactivate_item_popup(self, menu, view, old_selected):
        view.unselect_all()
        pass #print " *** Restoring previous selection"
        for path in old_selected:
            view.select_path(path)

    def _show_item_popup(self, view, ev):
        if ev.button == 3:
            path = view.get_path_at_pos(int(ev.x), int(ev.y))
            if path:
                model = view.get_model()
                item = model.get_value(model.get_iter(path), 2)
                if item:
                    old_selected = view.get_selected_items()

                    view.unselect_all()
                    view.select_path(path)

                    menu = gtk.Menu()
                    menu.attach_to_widget(view, None)
                    menu.connect("deactivate", self._deactivate_item_popup, view, old_selected)

                    pass #print " *** Showing item popup"
                    item.populate_popup(menu)
                    menu.popup(None, None, None, ev.button, ev.time)
                    return True

    def _item_drag_data_get(self, view, drag_context, selection_data, info, timestamp):
        # FIXME: Prefer ACTION_LINK if available
        if info == 100: # text/uri-list
            selected = view.get_selected_items()
            if not selected:
                return

            model = view.get_model()
            uris = []
            for path in selected:
                item = model.get_value(model.get_iter(path), 2)
                if not item:
                    continue
                uris.append(item.get_uri())

            pass #print " *** Dropping URIs:", uris
            selection_data.set_uris(uris)

    def _button_release(self, view, ev):
        if ev.button == 1 and self.click_policy == 'single':
            path = view.get_path_at_pos(int(ev.x), int(ev.y))
            if path:
                self.item_activated(path)
  
    def _motion_notify_timeout(self):
        x, y = self.get_pointer()
        path = self.get_path_at_pos(x, y)

        if path != self.highlight_path:
            if self.highlight_path:
                #self._undo_highlight()
                pass
            if path:
                #self._do_highlight(path)
                pass
            
        if self.click_policy == "single":
            cursor = None
            if path:
                # Set hand cursor
                cursor = gtk.gdk.Cursor(gtk.gdk.HAND2)
            self.window.set_cursor(cursor)

        self.highlight_timeout = 0
        del path,x,y
        return False

    def _motion_notify(self, view, ev):
        if self.highlight_timeout != 0:
            gobject.source_remove(self.highlight_timeout)

        # Avoid highlighting while rubberbanding
        if not ev.state & gtk.gdk.BUTTON1_MASK:
            ### FIXME: IconView flickers if we replace the icon too quickly, so use
            ###        a short timeout.
            self.highlight_timeout = gobject.timeout_add(5, self._motion_notify_timeout)

    def _click_policy_changed(self):
        self.click_policy = gconf_bridge.get("click_policy")

        if self.click_policy == "nautilus":
            self._connect_nautilus_click_policy()
        else:
            self._disconnect_nautilus_click_policy()


class ItemBox(gtk.VBox):
    def __init__(self):
        gtk.VBox.__init__(self)
        
        self.scrolled_window = gtk.ScrolledWindow()
        self.scrolled_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.pack_start(self.scrolled_window)
        self.show_all()
        
    def add_in_scroll(self, view):
        self.scrolled_window.show()
        self.scrolled_window.add_with_viewport(view)
        
class ItemButtonTable(gtk.VBox):
    def __init__(self):
        gtk.VBox.__init__(self)
        self.show_all()
        self.scrolled_window = gtk.ScrolledWindow()
        self.scrolled_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.pack_start(self.scrolled_window)
        
        self.scrolled_window.show()
        self.itembox= gtk.VBox()
        self.itembox.show()
        self.scrolled_window.add_with_viewport(self.itembox)

        
    def load_items(self,items):
        for item in self.itembox:
            self.itembox.remove(item)
            del item
        gc.collect()
        
        for item in items:
            btn = ItemButton(item)
            self.itembox.pack_start(btn,False,False)
    
class ItemButton(gtk.Button):
    def __init__(self,item):
        gtk.Button.__init__(self)
        self.item = item
        self.set_label(item.get_name())
        icon = item.get_icon(24)
        icon =icon_factory.load_image(icon,32)
        
        self.set_image(icon)
        self.show()
        self.set_relief(gtk.RELIEF_NONE)
        self.set_focus_on_click(False)
        self.set_alignment(0,0)
        self.connect("clicked",self.do_clicked)
        del item
        
    def do_clicked(self,btn):
        self.item.do_open()
        

class SidebarToggleButton(gtk.ToggleButton):
    '''
    A ToggleButton that represents an ItemSource.  Emits a "reload" signal when
    the button is toggled as active or if it\'s ItemSource emits "reload".
    The button contents will be the source\'s icon (large toolbar sized) and
    name, and the button cannot be untoggled by the user.
    '''
    __gsignals__ = {
        "reload" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())
        }

    def __init__(self, source):
        gtk.ToggleButton.__init__(self)
        self.source = source

        self.img = gtk.Image()
        pixbuf =source.get_icon(gtk.ICON_SIZE_SMALL_TOOLBAR)
        self.img.set_from_pixbuf(pixbuf)
        del pixbuf
        gc.collect()
        self.img.show()
        ### Uncomment to show the number of items in label
        #name = "%s (%d)" % (source.get_name(), len(source.get_items()))
        self.label = gtk.Label(source.get_name())
        self.label.show()

        self._reload_button_content()

        hbox = gtk.HBox(False, 4)
        hbox.pack_start(self.img, False, False, 0)
        hbox.pack_start(self.label, False, False, 0)
        hbox.show()

        self.add(hbox)
        self.set_focus_on_click(False)
        self.set_active(False)

        source.connect_after("reload", lambda s: self._reload_if_active())
        self.connect("toggled", lambda w: self._reload_if_active())
        
        
        self.visibile=True
        self.visiblebypref = True
        self.set_relief(gtk.RELIEF_NONE)
    
    def _reload_button_content(self):
        try:
            self.set_sensitive(self.source.get_enabled())
        except:
            pass
        pixbuf =self.source.get_icon(gtk.ICON_SIZE_LARGE_TOOLBAR)
        self.img.set_from_pixbuf(pixbuf)
        del pixbuf
        gc.collect()
        self.label.set_text(self.source.get_name())

    def _reload_if_active(self):
        self._reload_button_content()
        if self.get_active():
            self.emit("reload")

    

    def get_source(self):
        return self.source
        
class SidebarButton(gtk.Button):
    '''
    A ToggleButton that represents an ItemSource.  Emits a "reload" signal when
    the button is toggled as active or if it\'s ItemSource emits "reload".
    The button contents will be the source\'s icon (large toolbar sized) and
    name, and the button cannot be untoggled by the user.
    '''
    __gsignals__ = {
        "reload" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())
        }

    def __init__(self, source):
        gtk.Button.__init__(self)
        self.source = source

        self.img = gtk.Image()
        self.img.set_from_pixbuf(source.get_icon(8))
        self.img.show()
        ### Uncomment to show the number of items in label
        #name = "%s (%d)" % (source.get_name(), len(source.get_items()))
        self.label = gtk.Label(" "+source.get_name())
        self.label.show()
        self.label.set_ellipsize(pango.ELLIPSIZE_END)
        #self.label.set_justify(gtk.JUSTIFY_LEFT)

        self._reload_button_content()

        hbox = gtk.HBox(False,-1)
        hbox.pack_start(self.img, False, False, 0)
        hbox.pack_start(self.label, True, True, 1)
        hbox.show()
        self.set_size_request(-1,-1)
        self.add(hbox)
        self.set_focus_on_click(False)
        #self.set_active(False)

        source.connect_after("reload", lambda s: self._reload_if_active())
        self.connect("clicked", lambda w: self._reload_if_active())

        self.visibile=True
        self.visiblebypref = True
        self.set_relief(gtk.RELIEF_NONE)
        self.active = True
        del source
        gc.collect()
        
    def get_active(self):
        self.active
        
    def set_active(self,bool):
        self.active = bool
    
    def _reload_button_content(self):
        try:
            self.set_sensitive(self.source.get_enabled())
        except:
            pass
        self.img.set_from_pixbuf(self.source.get_icon(gtk.ICON_SIZE_LARGE_TOOLBAR))
        self.label.set_text(self.source.get_name())

    def _reload_if_active(self):
        self._reload_button_content()
        self.emit("reload")


    def get_source(self):
        return self.source
        
class Sidebar(gtk.VBox):
    '''
    A VBox containing SidebarButtons or spacers (empty-string labels).  Handles
    ensuring only one button is toggled at a time, and emits a "set-source"
    signal when a button toggled passing it\'s associated ItemSource.
    '''
    __gsignals__ = {
        "set-source" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
        "changed" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
        }

    def __init__(self, topic):
        gtk.VBox.__init__(self, False)
        self.btn_collection = []
        # FIXME: Not passed to __init__ to avoid binding bug #419107.
        self.set_spacing(0)

        self.topic = topic
        self.in_set_source = False

        # Listen for topic changes and reload the sidebar
        #topic.connect_after("reload", lambda t: self.topic_reload())
        topic.connect_after("reload", lambda t: self.add_sources())
        self.add_sources()

    def add_sources(self):
        
        for child in self.get_children():
            self.remove(child)
            
        active_source = None
        active_source_uri = None
        
        for child in self.get_children():
            if isinstance(child, SidebarButton) and child.get_active():
                active_source = child.get_source()
                active_source_uri = active_source.get_uri()
            self.remove(child)
            child.destroy()
        
        for source in self.topic.source_list:
            if not source:
                btn = gtk.Label("") # spacer
                btn.set_size_request(-1, 0)
                btn.show()
            elif source =="---":
                    btn = gtk.HSeparator() # separator
                    #btn.set_size_request(-1, 12)
                    btn.show()
            else:
            #try:
                    if source.needs_view:
                        btn = SidebarToggleButton(source)
                    else:
                        btn = SidebarButton(source)
                    btn.connect("reload", self._button_reload)
                    btn.connect("pressed", self._button_clicked)
                    btn.set_size_request(-1, -1)
                    
                    #btn.connect("enter", self._button_reload)
                    btn.show()
                    if source == active_source or  (active_source_uri and source.get_uri() == active_source_uri):
                                btn.set_active(True)
                    btn.show()
            
            self.pack_start(btn, False, False, 0)
            self.btn_collection.append(btn)
            '''                
            except:
                btn = gtk.Button(source) # spacer
                #btn.connect("clicked", self.reload_library_sources)
                btn.set_focus_on_click(False)
                btn.set_size_request(-1, -1)
                img = gtk.Image()
                btn.set_image(img)
                btn.show()
             '''   
         
        

    def _button_reload(self, btn):
        if not self.in_set_source: # Reentrance guard
            self.set_source(btn.get_source(),btn)
  
    def _button_clicked(self, btn):
        self.emit("changed",btn.get_source())
            
  
  
    def do_set_source(self, source):
        for btn in self:
            if isinstance(btn, SidebarToggleButton):
                # Setting the button active will cause a reload signal which
                # will emit a set-source signal.
                btn.set_active(btn.get_source() == source)

    def set_source(self, source, btn):
        self.in_set_source = True # Reentrance guard
        self.emit("set-source", source)
        self.in_set_source = False

    def topic_reload(self):
        self.emit("changed")
        pass

  
def color_average(col1, col2, colormap):
    if not col1 or not col2:
        return col1 or col2
    new = gtk.gdk.Color()
    new.red = int((col1.red + col2.red) / 2)
    new.green = int((col1.green + col2.green) / 2)
    new.blue = int((col1.blue + col2.blue) / 2)
    return colormap.alloc_color(new)

def color_average_widget_state(col1, widget, state):
    state_col = widget.get_style().bg[state]
    return color_average(col1, state_col, widget.get_colormap())        

class SearchToolItem(gtk.ToolItem):
    __gsignals__ = {
        "clear" : (gobject.SIGNAL_RUN_FIRST,
                   gobject.TYPE_NONE,
                   ()),
        "search" : (gobject.SIGNAL_RUN_FIRST,
                    gobject.TYPE_NONE,
                    (gobject.TYPE_STRING,))
        }

    def __init__(self, accel_group = None):
        gtk.ToolItem.__init__(self)
        self.search_timeout = 0
        self.default_search_text = _("Search")

        box = gtk.HBox(False, 0)
        box.show()

        try:
            img = icon_factory.load_image(gtk.STOCK_FIND, 16)
            img.show()
            ev_box = gtk.EventBox()
            ev_box.set_property('visible-window', False)
            ev_box.add(img)
            ev_box.show()

            self.clearbtn = gtk.EventBox()
            self.clearbtn.set_property('visible-window', False)
            self.clearbtn.set_size_request(-1, -1)
            self.clearbtn.show()
            self.clearbtn.connect("button-release-event", lambda w, ev: self.emit("clear"))

            self.iconentry = iconentry.IconEntry()
            self.iconentry.pack_widget(ev_box, True)
            self.iconentry.pack_widget(self.clearbtn, False)

            align = gtk.Alignment(0.5, 0.5)
            align.set_padding(0, 0, 0, 10)
            align.add(self.iconentry)
            align.show()

            box.pack_start(align, False, False, 0)
            self.entry = self.iconentry.get_entry()
        except NameError:
            self.clearbtn = None
            self.iconentry = None
            self.entry = gtk.Entry()
            box.pack_start(self.entry, True, True)

        self.entry.set_width_chars(14)
        self.entry.set_text(self.default_search_text)
        self.entry.show()
        self.entry.connect("activate", lambda w: self._typing_timeout())
        #self.entry.connect("focus-in-event", lambda w, x: self._entry_focus_in())
        self.entry.connect("key-press-event", self._entry_key_press)
        # Hold on to this id so we can block emission when initially clearing text
        self.change_handler_id = self.entry.connect("changed", lambda w: self._queue_search())
        '''
        if accel_group:
            # Focus on Ctrl-L
            self.entry.add_accelerator("grab-focus",
                                       accel_group,
                                       ord('l'),
                                       gtk.gdk.CONTROL_MASK,
                                       0)
        '''
        self.add(box)
        self.show_all()

    def do_clear(self):
        if self.clearbtn and self.clearbtn.child:
            self.clearbtn.remove(self.clearbtn.child)
        self._entry_clear_no_change_handler()

    def do_search(self, text):
        if self.clearbtn and not self.clearbtn.child:
            img = icon_factory.load_image(gtk.STOCK_CLOSE, 16)
            img.show()
            self.clearbtn.add(img)

    def _entry_clear_no_change_handler(self):
        '''Avoids sending \'changed\' signal when clearing text.'''
        self.entry.handler_block(self.change_handler_id)
        self.entry.set_text("")
        self.entry.handler_unblock(self.change_handler_id)

    def _entry_focus_in(self):
        '''Clear default search text'''
        print ("Clear default search text")
        if self.entry.get_text() == self.default_search_text:
            self._entry_clear_no_change_handler()

    def _typing_timeout(self):
        if len(self.entry.get_text()) > 0:
            self.emit("search", self.entry.get_text())
        self.search_timeout = 0
        return False

    def _queue_search(self):
        if self.search_timeout != 0:
            gobject.source_remove(self.search_timeout)
            self.search_timeout = 0

        if len(self.entry.get_text()) == 0:
            self.emit("clear")
        else:
            self.search_timeout = gobject.timeout_add(50, self._typing_timeout)

    def _entry_key_press(self, w, ev):
        if ev.keyval == gtk.gdk.keyval_from_name("Escape") \
               and len(self.entry.get_text()) > 0:
            self.emit("clear")
            return True

    def get_search_text(self):
        if self.entry.get_text() == self.default_search_text:
            return None
        return self.entry.get_text()

    def cancel(self):
        '''Cancel a pending/active search without sending the \'clear\' signal.'''
        if self.entry.get_text() != self.default_search_text:
            self.do_clear()

class TopicWidgetConnector(gtk.VBox):
    __gsignals__ = {
        "set-source" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,)),
        "activate" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        "deactivate" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        }
    
    def __init__(self,topic):
        gtk.VBox.__init__(self)
        self.active=False
        self.btn =  TopicButtonMod(topic)
        self.topic = topic
        self.btn.show()
        
        self.pack_start(self.btn,False,False)
        self.sidebar = Sidebar(self.topic)
        #self.sidebar.connect("set-source",self. set_source)
        self.sidebar.connect("changed",self. set_source)
        
        self.btn.connect_after("pressed",self.activate)
        self.show()
        
    def set_source(self,sidebar,source):
        if self.active:
            if source.needs_view:
                print("boooooooogoooooooo")
                self.emit("set-source",source)
            else:
                source.do_open()
        
    def activate(self,btn):
        btnName = self.btn.name
        if not self.active:
            self.emit('activate')
            self.show_sidebar()
        else:
            self.emit('deactivate')
            self.hide_sidebar()
    
        gc.collect()
            
    def show_sidebar(self):
        btnName = self.btn.name
        self.active=True
        self.sidebar.show() 
        gc.collect()
        


            
    def hide_sidebar(self):
        btnName = self.btn.name
        self.active=False
        self.sidebar.hide()
        gc.collect()
        

            
    def get_sidebar(self):
        return self.sidebar
        

class MayannaWidget(gtk.EventBox):
    def __init__(self):
        gtk.EventBox.__init__(self)
        
        self.topicWindow = None
        
        apps = ApplicationsTopic()
        lib = DocumentsTopic()
        ppl = PeopleTopic()
        cmp = ComputerTopic()
        fav = FavoritesTopic()
        
        self.active_source = None
        self.time_filter = 7
        self.show_status = False
        self.topics = [fav,apps,lib,ppl,cmp]
        
        '''
        Topics Buttons Box
        '''
        
        
        self.vbox1 =  gtk.VBox()
        self.topicTable = gtk.HBox()
        self.topicBox =  gtk.VBox()
        
        
        
        
        
        self.evbox2 = gtk.EventBox()
        self.evbox2.add(self.vbox1)
        self.evbox2.set_visible_window(True)
        self.evbox2.set_border_width(1)
        self.evbox2.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
        
        self.evbox = gtk.EventBox()
        self.evbox.add(self.evbox2)
        self.evbox.set_visible_window(True)
        self.evbox.set_border_width(5)
        self.evbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("black"))
        self.add(self.evbox)
        
        '''
        Set up search tool
        ''' 
        
        self.accel_group = gtk.AccelGroup()
        self.search_tool_item = SearchToolItem(self.accel_group)
        #self.search_tool_item.set_tooltip(self.tooltips, ("Search"))
        self.search_tool_item.connect("search", self._search)
        self.search_tool_item.connect("clear", self._search_clear)
        self.searchbox = gtk.VBox()
        self.searchbox.pack_start(self.search_tool_item)
        
        self.search_tool_item.entry.grab_focus()
        
        '''
        Set up Topic buttons
        '''
        
        self.searchTopicBox= gtk.VBox()
        self.searchTopicBox.pack_start(self.searchbox)
        self.topicButtonBox = gtk.VBox()
        self.sidebarBox = gtk.VBox()
        
        self.frame1 = gtk.Frame()
        self.alignment1 = gtk.Alignment(0.5,0.5,1.0,1.0)
        
        self.label1 = gtk.Label()
        self.sidebarBtnBox = gtk.VBox()
        self.connectors = []
        
        
        self.scrolled_window = gtk.ScrolledWindow()
        self.scrolled_window.set_border_width(4)
        #self.scrolled_window.set_size_request(-1,300)
        #self.scrolled_window.set_resizable(True)

        self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scrolled_window.add_with_viewport(self.sidebarBtnBox)
        self.scrolled_window.show()

        
       
        
        self.sidebarBox.hide()
        for topic in  self.topics:
    
            box = TopicWidgetConnector(topic)    
            #box.show()
            box.connect("activate",self.toggle)
            box.connect("deactivate",self.hide_sidebarBtnBox)
            box.connect("set-source",self.load_items)
            #self.topicButtonBox.pack_start(box)
            self.connectors.append(box)
        gc.collect()
        
        ################ temp #################
        hbox1 = gtk.Table(2,2,True)
        hbox1.attach(self.connectors[1],0,1,0,1)
        hbox1.attach(self.connectors[2],1,2,0,1)
        hbox1.attach(self.connectors[3],0,1,1,2)
        hbox1.attach(self.connectors[4],1,2,1,2)
        hbox1.set_border_width(1)
        
        tempBox = gtk.VBox(False,5)
        tempBox.pack_start(self.connectors[0],True,True)
        tempBox.pack_start(hbox1,True,True)
        
        #self.evbox3 = gtk.EventBox()
        self.frame2 = gtk.Frame()
        
        self.frame2.add(tempBox)
        self.frame2.show()
        
        ######################################
        
        self.vbox1.pack_start(self.topicTable,True,True)
        self.windowListBox = gtk.HBox()
        self.vbox1.pack_start(self.windowListBox,True,True)
        self.topicTable.pack_start(self.topicBox,True,True,5)
        self.topicBox.pack_start(self.searchTopicBox,True,True,5)
        self.topicBox.pack_start(self.frame2,True,True)
        self.topicBox.pack_start(self.sidebarBox,True,True)
        self.sidebarBox.pack_start(self.frame1,True,True,5)
        #self.frame1.set_shadow_type(gtk.SHADOW_OUT)
        #self.sidebarBox.pack_start(gtk.Button("xxxxx"),True,True,5)
        self.frame1.set_label_align(0.5, 0.5)
        self.frame1.add(self.alignment1)
        self.frame1.set_label_widget(self.label1)
        self.alignment1.add(self.scrolled_window)
        '''
         Viewer to view Items
         '''
        self.viewWindow = gtk.Window()
        self.viewWindow.connect("focus-out-event",self.lose_focus)
        type = gtk.gdk.WINDOW_TYPE_HINT_MENU
        self.viewWindow.set_type_hint(type)
        self.viewWindow.set_decorated(False)
        self.viewWindow.set_skip_pager_hint(False)
        self.viewWindow.set_skip_taskbar_hint(False)
        
                
        self.eventbox3 = gtk.EventBox()
        self.eventbox3.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("black"))
        self.eventbox4 = gtk.EventBox()
        #self.eventbox4.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("grey"))
        
        self.eventbox3.set_visible_window(True)
        self.eventbox3.set_border_width(1)
        
        self.eventbox4.set_visible_window(True)
        self.eventbox4.set_border_width(1)
        
        self.viewBox = gtk.VBox()
        
        # Scrolled window containing th be icon view
        
        self.timeLineBox = gtk.VBox()
        self.viewItemsBox = gtk.VBox()
        
        
        #self.view = ItemButtonTable()
        self.view = ItemIconView()
        self.view.set_size_request(-1,-1)
        self.view.show()
        
        self.viewscroll = ItemBox()
        self.viewscroll.add_in_scroll(self.view)
        
        
        self.viewWindow.add(self.eventbox3)
        self.eventbox3.add(self.eventbox4)
        self.eventbox4.add(self.viewBox)
        self.viewBox.pack_start(self.timeLineBox,False,True,3)
        
        self.viewBox.pack_start(self.viewscroll,True,True)
        '''
         Day Zoom tool
         '''
         #self.timeLineBox = self.wTree.get_widget("timeLineBox")
        self.timebar = TimeBar()
        self.timebar.connect("zoom-changed", lambda w, num_days: self.zoom_changed(num_days))
        self.timeLineBox.pack_start(self.timebar)
         #self.timebar.show()
               
       
        self.results = []
        
        self.viewWindow.show_all()
        self.viewWindow.hide()
        self.connectors[0].activate(None)
        self.search_tool_item.entry.grab_focus()
        self.vbox1.connect("focus-in-event",self.get_focus)
  
    def set_parent_size(self,topicWindow):
          self.topicWindow=topicWindow
          self.applet = gnomeapplet.Applet()
        #self.add_favorite_toolitem()
          AppletOrientationHelper(self.viewWindow,self.applet,self.topicWindow) 
          
          self.viewWindow.set_resizable(True)
          self.viewWindow.resize(200,(self.topicWindow.get_screen().get_height())/1.3)
      
    def lose_focus(self,source1, source2):
        self.viewWindow.hide()
        
    def get_focus(self,x,y):
        self.search_tool_item.entry.grab_focus()
                 
    def hide_sidebarBtnBox(self,btn):
        self.sidebarBox.hide()
        self.viewWindow.hide()
        #self.windowListBox.show()
           
    def set_presenter(self,presenter):
        self.presenter=presenter
        
    '''   GUI functionality '''
    
    def load_items(self,sidebar,source):
        print("xxxxxxxxxxxxxxxx")
        if source.needs_view:
            self.search_tool_item.entry.set_text("")
            gc.collect()
            #Check if source wants date filter
            filter = source.get_filter_by_date()
            if filter:
                self.show_hide_zoomer(True)
                items = self._filter_items_num_days_ago(source.get_items(),self.time_filter)                    
                self.view.load_items(items)
                del items
                gc.collect()
            else:
                self.show_hide_zoomer(False)
                items = source.get_items()
                self.view.load_items(items)
                del items
                gc.collect()
                
            self.active_source = source
            
            gc.collect()
            self.set_parent_size(self.topicWindow)
            self.viewWindow.show()
        else:
            self.viewWindow.hide()
            
    def toggle(self,btn):
        self.viewWindow.hide()
        for box in self.connectors:
            if not box.btn == btn.btn:
                box.hide_sidebar() 
                self.sidebarBox.hide() 
                box.btn.set_active(False)
                #box.btn.set_active(False)
        self.sidebarBox.show()
        label="<b>  " + btn.btn.name + "  </b>"
        self.label1.set_markup(label)
        self.label1.UseMarkup = True;
        #self.sidebarBtnBox = self.wTree.get_widget("sidebarBtnBox")
        self.sidebar = btn.get_sidebar()
        #btn.btn.set_active(True)
        '''
        butcount = len(self.sidebar.get_children())
        print("---------------------" + str(butcount))
        count = butcount * 36
        print("++++++++" + str(butcount))
       '''
        x,y=self.sidebar.size_request()
        
        count = y +32
        print("++++++++" + str(count))
        if count < 300:
            self.scrolled_window.set_size_request(-1,count )
        else:
            self.scrolled_window.set_size_request(-1,300)
        #self.windowListBox.hide()
        self.sidebar.show()
        print("xxxxx")
        self.sidebarBtnBox.pack_start(btn.get_sidebar(),True,True)
        print("xxxxx")
        gc.collect()
            
    '''   Zoom functionality  '''
    
    def zoom_changed(self, num_days):
        self.time_filter = num_days
        if self.active_source:
            filtered = self._filter_items_num_days_ago(self.active_source.get_items(),  self.time_filter)
            # FIXME: This loses the focused item
            print("---------------------------------------__")
            self.view.load_items(filtered)
            del filtered
        gc.collect()

    def _filter_compare(self, i, days_ago):
        return i.is_special() or datetime.date.fromtimestamp(i.get_timestamp()) > days_ago

    def _filter_items_num_days_ago(self, items, num_days):
        if num_days < 0:
            days_ago = datetime.date.min
        else:
            days_ago = datetime.date.today() - datetime.timedelta(days = num_days)

        return [x for x in items if self._filter_compare(x, days_ago)]
            
    def show_hide_zoomer(self,show):
            if show:
                self.timebar.show()
            else:
                self.timebar.hide()
           
    '''  Search functions '''
     
    def _search(self, w, text):
        self.viewWindow.show()
    
        self.results = []
        for topic in self.topics:
            if not topic.name == "Favorites":
                matches = topic.find_items(text)
                for item in matches:
                     self.results.append(item)
                #results = results +  [matches]
        self.load_search_items(self.results)
        #collector.delete_refs(results)
        for i in self.results:
            del i
        for box in self.connectors:
                box.hide_sidebar()
                del box
        #collector.delete_refs(results)
        
        gc.collect()
    
    def _search_clear(self, w):
        self.view.load_items([])
        gc.collect()
        self.viewWindow.hide()
    
    def load_search_items(self,results):
            self.sidebarBox.hide()
            self.view.load_items(results)
            #collector.delete_refs(results)
           # delresults 
            gc.collect()
            
    '''  Favorite Tool Button '''  
     
    def _favorite_selection_changed(self, view, fav, block_id, img):
        # Block the toggled handler from triggering
        fav.handler_block(block_id)
        selected = view.get_selected_items()
        if len(selected) == 1:
            model = view.get_model()
            item = model.get_value(model.get_iter(selected[0]), 2)

            pix = icon_factory.load_icon("gnome-favorites", gtk.ICON_SIZE_LARGE_TOOLBAR)
            if not item.get_is_pinned():
                pix = icon_factory.greyscale(pix)
            img.set_from_pixbuf(pix)
            del pix
            gc.collect()
        
            fav.set_active(item.get_is_pinned())
            fav.set_sensitive(item.get_can_pin())
        else:
            fav.set_active(False)
            fav.set_sensitive(False)
        fav.handler_unblock(block_id)

    def _favorite_toggled(self, fav):
        selected = self.view.get_selected_items()
        if len(selected) == 1:
            model = self.view.get_model()
            item = model.get_value(model.get_iter(selected[0]), 2)
            if fav.get_active():
                item.pin()
            else:
                item.unpin()

    def add_favorite_toolitem(self):
        img = icon_factory.load_image("gnome-favorites", gtk.ICON_SIZE_LARGE_TOOLBAR)
        
        i = gtk.ToggleToolButton()
        i.set_label(("Favorite"))
        i.set_icon_widget(img)
        i.set_is_important(True)
        i.set_sensitive(False)
        i.set_tooltip(self.tooltips,("Add to Favorites"))
        i.show_all()
        self.toolbar.insert(i, -1)
        
        block_id = i.connect("toggled", self._favorite_toggled)
        
        sel_changed_cb = lambda: self._favorite_selection_changed(self.view, i, block_id, img)
        #self.view.connect("selection-changed", lambda view: sel_changed_cb())
        bookmarks.connect("reload", lambda b: sel_changed_cb())

        self.fav = i
   
    def get_widget(self):
        return self.evbox

class AppletOrientationHelper:
    def __init__(self, widget, applet, track_widget):
        update_cb = lambda: self.update_position(widget, applet, track_widget)

        widget.connect("size-allocate", lambda w, a: update_cb())
        track_widget.connect("size-allocate", lambda w, a: update_cb())
        track_widget.connect("screen-changed",
                             lambda w, screen: widget.set_screen(screen))

    def update_position(self, widget, applet, track_widget):
        '''
        Calculates position and moves window to it.
        '''
        if not track_widget.flags() & gtk.REALIZED:
            return

        # Get our own dimensions & position        
        xs, ys = track_widget.get_size()
        xs2, ys2 = widget.get_size()
        
        
        geom = track_widget.get_screen().get_monitor_geometry(0)
        tempx = geom.width - xs- xs2
        
        x = tempx
        y = ys/2 - ys2/2
        widget.move(x, y)
        
        
class TimeBar(gtk.EventBox):
    __gsignals__ = {

        "zoom-changed" : (gobject.SIGNAL_RUN_LAST,
                          gobject.TYPE_NONE,
                          (gobject.TYPE_INT,)),
        }

    _zoomlist = [[_("Today"), 1],
                 [_("This week"), 7],
                 [_("Last 2 weeks"), 14],
                 [_("This month"), 30],
                 [_("Last 2 months"), 60],
                 [_("Last 3 months"), 90],
                 [_("Last 6 months"), 180]]
    _default_zoom = 7

    def __init__(self):
        gtk.EventBox.__init__(self)
        #self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("grey"))
        
        box = gtk.HBox(True)
        box.show()
        self.add(box)
        #self._add_sparkline(box)

        self.label = gtk.Label()
        #self.label.set_width_chars(max([len(x[0]) for x in self._zoomlist]))
        #self.label.set_alignment(1.0, 0.5)
       
        self.in_btn = gtk.Button()
        self.in_btn.add(gtk.image_new_from_stock(gtk.STOCK_REMOVE, gtk.ICON_SIZE_LARGE_TOOLBAR))
        self.in_btn.set_relief(gtk.RELIEF_NONE)
        self.in_btn.connect("clicked", lambda btn: self._zoom_clicked(-1))
        
        self.out_btn = gtk.Button()
        self.out_btn.add(gtk.image_new_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_LARGE_TOOLBAR))
        self.out_btn.set_relief(gtk.RELIEF_NONE)
        self.out_btn.connect("clicked", lambda btn: self._zoom_clicked(1))
        
        box.pack_start(self.in_btn, True, True)
        box.pack_start(self.label, True, True)
        box.pack_start(self.out_btn, True, True)

        box.show_all()

        self.set_zoom_level(self._default_zoom)

    def _add_sparkline(self, box):
        self.spark = None
        
        ### Disabled due to bugginess for 0.2.8
        #self.spark = SparkLine(self.get_zoom_level_list(floor=0))
        #self.spark.connect("zoom-changed", lambda w, num_days: self.set_zoom_level(num_days))

        #align = gtk.Alignment(0.5, 0.5, 1.0, 1.0)
        #align.set_property("top-padding", 4)
        #align.add(self.spark)
        #box.pack_start(align, True, True, 6)

    def get_zoom_level(self):
        return self.zoom_level

    def get_zoom_level_list(self, floor = _default_zoom):
        return [days for name, days in self._zoomlist if days >= floor]

    def _zoom_clicked(self, dir):
        for i in range(len(self._zoomlist)):
            label, days = self._zoomlist[i]
            if days == self.zoom_level:
                try:
                    label, days = self._zoomlist[i + dir]
                except IndexError:
                    label, days = self._zoomlist[0]
                self.set_zoom_level(days, label)
                break

    def set_zoom_level(self, zoom_level, label=None):
        self.zoom_level = zoom_level

        if not label:
            for _label, days in self._zoomlist:
                if days == zoom_level:
                    label = _label
        self.label.set_text(label)
        self.emit("zoom-changed", zoom_level)
        
    def set_items(self, items):
        dates = [x.get_timestamp() for x in items if x.get_timestamp() != 0]
        dates = [datetime.date.fromtimestamp(x) for x in dates]
